@@ -49,7 +49,10 @@ ignored_files = {
   "runtest.c": "regression tests program",
   "runsuite.c": "regression tests program",
   "tst.c": "not part of the library",
+  "test.c": "not part of the library",
   "testdso.c": "test for dynamid shared libraries",
+  "testrecurse.c": "test for entities recursions",
+  "xzlib.h": "Internal API only",
 }
 
 ignored_words = {
@@ -63,12 +66,17 @@ ignored_words = {
   "EXSLTPUBFUN": (0, "Special macro for extern funcs for win32"),
   "XMLCALL": (0, "Special macro for win32 calls"),
   "XSLTCALL": (0, "Special macro for win32 calls"),
+  "XMLCDECL": (0, "Special macro for win32 calls"),
   "EXSLTCALL": (0, "Special macro for win32 calls"),
   "__declspec": (3, "Windows keyword"),
   "__stdcall": (0, "Windows keyword"),
   "ATTRIBUTE_UNUSED": (0, "macro keyword"),
   "LIBEXSLT_PUBLIC": (0, "macro keyword"),
   "X_IN_Y": (5, "macro function builder"),
+  "ATTRIBUTE_ALLOC_SIZE": (3, "macro for gcc checking extension"),
+  "ATTRIBUTE_PRINTF": (5, "macro for gcc printf args checking extension"),
+  "LIBXML_ATTR_FORMAT": (5, "macro for gcc printf args checking extension"),
+  "LIBXML_ATTR_ALLOC_SIZE": (3, "macro for gcc checking extension"),
 }
 
 def escape(raw):
@@ -354,7 +362,7 @@ class index:
 	self.analyze_dict("structs", self.structs)
 	self.analyze_dict("typedefs", self.typedefs)
 	self.analyze_dict("macros", self.macros)
-         
+
 class CLexer:
     """A lexer for the C language, tokenize the input by reading and
        analyzing it line by line"""
@@ -386,7 +394,7 @@ class CLexer:
 		else:
 		    line = line + n
         return line
-	 
+
     def getlineno(self):
         return self.lineno
 
@@ -555,12 +563,12 @@ class CLexer:
 		    else:
 		        break
 		self.tokens.append(('name', line[s:i]))
-             
+
 	tok = self.tokens[0]
 	self.tokens = self.tokens[1:]
 	self.last = tok
 	return tok
-      
+
 class CParser:
     """The C module parser"""
     def __init__(self, filename, idx = None):
@@ -678,9 +686,54 @@ class CParser:
 
 	return token
 
-     #
-     # Parse a comment block associate to a macro
-     #
+    #
+    # Parse a comment block associate to a typedef
+    #
+    def parseTypeComment(self, name, quiet = 0):
+        if name[0:2] == '__':
+	    quiet = 1
+
+        args = []
+	desc = ""
+
+        if self.comment == None:
+	    if not quiet:
+		self.warning("Missing comment for type %s" % (name))
+	    return((args, desc))
+        if self.comment[0] != '*':
+	    if not quiet:
+		self.warning("Missing * in type comment for %s" % (name))
+	    return((args, desc))
+	lines = string.split(self.comment, '\n')
+	if lines[0] == '*':
+	    del lines[0]
+	if lines[0] != "* %s:" % (name):
+	    if not quiet:
+		self.warning("Misformatted type comment for %s" % (name))
+		self.warning("  Expecting '* %s:' got '%s'" % (name, lines[0]))
+	    return((args, desc))
+	del lines[0]
+	while len(lines) > 0 and lines[0] == '*':
+	    del lines[0]
+	desc = ""
+	while len(lines) > 0:
+	    l = lines[0]
+	    while len(l) > 0 and l[0] == '*':
+	        l = l[1:]
+	    l = string.strip(l)
+	    desc = desc + " " + l
+	    del lines[0]
+
+	desc = string.strip(desc)
+
+	if quiet == 0:
+	    if desc == "":
+	        self.warning("Type comment for %s lack description of the macro" % (name))
+
+	return(desc)
+    #
+    # Parse a comment block associate to a macro
+    #
     def parseMacroComment(self, name, quiet = 0):
         if name[0:2] == '__':
 	    quiet = 1
@@ -740,7 +793,7 @@ class CParser:
 	    l = string.strip(l)
 	    desc = desc + " " + l
 	    del lines[0]
-		     
+
 	desc = string.strip(desc)
 
 	if quiet == 0:
@@ -841,7 +894,7 @@ class CParser:
 	    else:
 	        desc = desc + " " + l
 		del lines[0]
-		     
+
 	retdesc = string.strip(retdesc)
 	desc = string.strip(desc)
 
@@ -851,7 +904,8 @@ class CParser:
 	     #
 	    i = 0
 	    while i < nbargs:
-	        if args[i][2] == None and args[i][0] != "void" and args[i][1] != None:
+	        if args[i][2] == None and args[i][0] != "void" and \
+		   ((args[i][1] != None) or (args[i][1] == '')):
 		    self.warning("Function comment for %s lacks description of arg %s" % (name, args[i][1]))
 		i = i + 1
 	    if retdesc == "" and ret[0] != "void":
@@ -859,7 +913,6 @@ class CParser:
 	    if desc == "":
 	        self.warning("Function comment for %s lacks description of the function" % (name))
 
-
 	return(((ret[0], retdesc), args, desc))
 
     def parsePreproc(self, token):
@@ -919,7 +972,7 @@ class CParser:
 	    try:
 	        self.defines.append(apstr)
 		if string.find(apstr, 'ENABLED') != -1:
-		    self.conditionals.append("defined(%s)" % apstr)    
+		    self.conditionals.append("defined(%s)" % apstr)
 	    except:
 	        pass
 	elif name == "#ifndef":
@@ -927,7 +980,7 @@ class CParser:
 	    try:
 	        self.defines.append(apstr)
 		if string.find(apstr, 'ENABLED') != -1:
-		    self.conditionals.append("!defined(%s)" % apstr)    
+		    self.conditionals.append("!defined(%s)" % apstr)
 	    except:
 	        pass
 	elif name == "#if":
@@ -1024,8 +1077,10 @@ class CParser:
 					"struct", type)
 			base_type = "struct " + name
 	            else:
+			# TODO report missing or misformatted comments
+			info = self.parseTypeComment(name, 1)
 			self.index_add(name, self.filename, not self.is_header,
-		                    "typedef", type)
+		                    "typedef", type, info)
 		token = self.token()
 	    else:
 		self.error("parsing typedef: expecting a name")
@@ -1047,7 +1102,7 @@ class CParser:
 		return token
 	token = self.token()
 	return token
-	     
+
      #
      # Parse a C code block, used for functions it parse till
      # the balancing } included
@@ -1082,7 +1137,7 @@ class CParser:
 		    elif oldtok[0] == "name" and oldtok[1][0:7] == "LIBXML_":
 			self.index_add_ref(oldtok[1], self.filename,
 					    0, "typedef")
-			 
+
 		else:
 		    token = self.token()
 	return token
@@ -1215,7 +1270,7 @@ class CParser:
 	if token == None:
 	    return token
 
-	while token[0] == "name" and ( 
+	while token[0] == "name" and (
 	      token[1] == "const" or \
 	      token[1] == "unsigned" or \
 	      token[1] == "signed"):
@@ -1235,7 +1290,7 @@ class CParser:
 		    self.type = tmp[1]
 		else:
 		    self.type = self.type + " " + tmp[1]
-	     
+
         elif token[0] == "name" and token[1] == "struct":
 	    if self.type == "":
 	        self.type = token[1]
@@ -1525,7 +1580,7 @@ class CParser:
 			token = self.token()
 		else:
 		    break
-		    
+
 	return token
 
     def parse(self):
@@ -1541,7 +1596,7 @@ class CParser:
 		return
 	self.parseTopComment(self.top_comment)
         return self.index
-	         
+
 
 class docBuilder:
     """A documentation builder"""
@@ -1638,7 +1693,7 @@ class docBuilder:
 		    self.headers[file] = None;
 	self.scanHeaders()
 	self.scanModules()
-         
+
     def modulename_file(self, file):
         module = os.path.basename(file)
 	if module[-2:] == '.h':
@@ -1712,8 +1767,17 @@ class docBuilder:
 	    else:
 	        output.write("/>\n");
 	else :
-	    output.write("    <typedef name='%s' file='%s' type='%s'/>\n" % (
-	             name, self.modulename_file(id.header), id.info))
+	    output.write("    <typedef name='%s' file='%s' type='%s'" % (
+	                 name, self.modulename_file(id.header), id.info))
+            try:
+		desc = id.extra
+		if desc != None and desc != "":
+		    output.write(">\n      <info>%s</info>\n" % (escape(desc)))
+		    output.write("    </typedef>\n")
+		else:
+		    output.write("/>\n")
+	    except:
+		output.write("/>\n")
 
     def serialize_variable(self, output, name):
         id = self.idx.variables[name]
@@ -1723,7 +1787,7 @@ class docBuilder:
 	else:
 	    output.write("    <variable name='%s' file='%s'/>\n" % (
 	            name, self.modulename_file(id.header)))
-	              
+
     def serialize_function(self, output, name):
         id = self.idx.functions[name]
 	if name == debugsym:
@@ -1744,6 +1808,11 @@ class docBuilder:
 	    output.write("      <cond>%s</cond>\n"% (apstr));
 	try:
 	    (ret, params, desc) = id.info
+	    if (desc == None or desc == '') and \
+	       name[0:9] != "xmlThrDef" and name != "xmlDllMain":
+	        print "%s %s from %s has no description" % (id.type, name,
+		       self.modulename_file(id.module))
+
 	    output.write("      <info>%s</info>\n" % (escape(desc)))
 	    self.indexString(name, desc)
 	    if ret[0] != None:
