@@ -80,7 +80,7 @@ testXmlwriterFilename(const char *uri)
     }
 
     /* Start the document with the xml default for the version,
-     * encoding ISO 8858-1 and the default for the standalone
+     * encoding ISO 8859-1 and the default for the standalone
      * declaration. */
     rc = xmlTextWriterStartDocument(writer, NULL, MY_ENCODING, NULL);
     if (rc < 0) {
@@ -160,7 +160,7 @@ testXmlwriterFilename(const char *uri)
 
     /* Write an element named "X_ORDER_ID" as child of HEADER. */
     rc = xmlTextWriterWriteFormatElement(writer, BAD_CAST "X_ORDER_ID",
-                                         "%010d", 53535L);
+                                         "%010d", 53535);
     if (rc < 0) {
         printf
             ("testXmlwriterFilename: Error at xmlTextWriterWriteFormatElement\n");
@@ -359,7 +359,7 @@ testXmlwriterMemory(const char *file)
     }
 
     /* Start the document with the xml default for the version,
-     * encoding ISO 8858-1 and the default for the standalone
+     * encoding ISO 8859-1 and the default for the standalone
      * declaration. */
     rc = xmlTextWriterStartDocument(writer, NULL, MY_ENCODING, NULL);
     if (rc < 0) {
@@ -439,7 +439,7 @@ testXmlwriterMemory(const char *file)
 
     /* Write an element named "X_ORDER_ID" as child of HEADER. */
     rc = xmlTextWriterWriteFormatElement(writer, BAD_CAST "X_ORDER_ID",
-                                         "%010d", 53535L);
+                                         "%010d", 53535);
     if (rc < 0) {
         printf
             ("testXmlwriterMemory: Error at xmlTextWriterWriteFormatElement\n");
@@ -468,7 +468,7 @@ testXmlwriterMemory(const char *file)
     /* Write an element named "NAME_2" as child of HEADER. */
     tmp = ConvertInput("Jrg", MY_ENCODING);
     rc = xmlTextWriterWriteElement(writer, BAD_CAST "NAME_2", tmp);
-                                   
+
     if (rc < 0) {
         printf
             ("testXmlwriterMemory: Error at xmlTextWriterWriteElement\n");
@@ -636,7 +636,7 @@ testXmlwriterDoc(const char *file)
     }
 
     /* Start the document with the xml default for the version,
-     * encoding ISO 8858-1 and the default for the standalone
+     * encoding ISO 8859-1 and the default for the standalone
      * declaration. */
     rc = xmlTextWriterStartDocument(writer, NULL, MY_ENCODING, NULL);
     if (rc < 0) {
@@ -709,7 +709,7 @@ testXmlwriterDoc(const char *file)
 
     /* Write an element named "X_ORDER_ID" as child of HEADER. */
     rc = xmlTextWriterWriteFormatElement(writer, BAD_CAST "X_ORDER_ID",
-                                         "%010d", 53535L);
+                                         "%010d", 53535);
     if (rc < 0) {
         printf
             ("testXmlwriterDoc: Error at xmlTextWriterWriteFormatElement\n");
@@ -908,7 +908,7 @@ testXmlwriterTree(const char *file)
     }
 
     /* Start the document with the xml default for the version,
-     * encoding ISO 8858-1 and the default for the standalone
+     * encoding ISO 8859-1 and the default for the standalone
      * declaration. */
     rc = xmlTextWriterStartDocument(writer, NULL, MY_ENCODING, NULL);
     if (rc < 0) {
@@ -975,7 +975,7 @@ testXmlwriterTree(const char *file)
 
     /* Write an element named "X_ORDER_ID" as child of HEADER. */
     rc = xmlTextWriterWriteFormatElement(writer, BAD_CAST "X_ORDER_ID",
-                                         "%010d", 53535L);
+                                         "%010d", 53535);
     if (rc < 0) {
         printf
             ("testXmlwriterTree: Error at xmlTextWriterWriteFormatElement\n");
@@ -1168,8 +1168,8 @@ ConvertInput(const char *in, const char *encoding)
     if (out != 0) {
         temp = size - 1;
         ret = handler->input(out, &out_size, (const xmlChar *) in, &temp);
-        if (ret || temp - size + 1) {
-            if (ret) {
+        if ((ret < 0) || (temp - size + 1)) {
+            if (ret < 0) {
                 printf("ConvertInput: conversion wasn't successful.\n");
             } else {
                 printf
